% function x = macoptII(start,tol,verbose,itmax,typdist,gradfunction[,gradarg1[,gradarg2[,...]]])
% 
% Bare-bones mex interface to David MacKay's macopt gradient-only optimizer.
% 
% Mandatory Inputs:
%        start  Dx1     vector initial condition for optimzer
%          tol  1x1     eg 0.001, stop if |gradient vector|<tol or <|step|> < tol
%      verbose  1x1     eg 0, Verbosity level from 0 (silent) to 4 (very verbose)
%        itmax  1x1     eg 100, Maximum number of iterations to run
%      typdist  1x1     eg 0.001, typical distance at which the line minimum is expected
% gradfunction  string  string containing function to evaluate gradients
%
% Optional Inputs:
%     gradarg1  ???     Any number of additional arguments of any type may be
%     gradarg2  ???     specified. These will all be passed on to gradfunction
%       ...     ...     every time it is called bymacopt.
% 
% Outputs:
%            x  Dx1     minimum point of function
%
% Example usage:
% 	opt=macoptII([1;1],1e-5,0,100,0.01,'gradfn',[1 2;2 1],1)
%
% This mex interface is by Iain Murray, http://www.gatsby.ucl.ac.uk/~iam23/ June
% 2004. It doesn't expose all of the options available in macopt. I figured a
% typical Matlab user wants something simple. Advanced users can recompile the
% interface with additional options: source and a Makefile is provided.
% Similarly I haven't yet wrapped the covariant version but that could be added
% easily.
% 
% More information about the macopt optimiser itself is below and at the
% following website:
% 	http://www.inference.phy.cam.ac.uk/mackay/c/macopt.html
% 
% If you find macopt useful, please feel free to make a donation to support
% David MacKay's research group.
% 
% *** THIS CODE COMES WITH NO WARRANTY OF ANY KIND. ***
% 
%  The following important information is copy-and-pasted from the C-source code:
%  ------------------------------------------------------------------------------
%
%  Please do not use macopt without understanding a little about how it works;
%  there are some control parameters which the user MUST set!
%
%  This optimizer should only be used for functions whose first derivative
%  is continuous.
%
%  David MacKay's optimizer, based on conjugate gradient ideas, 
%  but using bracketing of the zero of the inner product 
%
%            (gradient).(line_search_direction)
%
%  to do the line minimization. Only derivative calculations are required.
%  The length of the first step in the line search (often set to "1.0"
%  in other code) is adapted here so that, if 0.00001 is a better step size, 
%  it soon cottons on to that and saves ~log(10000) bracketing operations.
%  The result is that (with 'rich' set to 0) the program can use 
%  as few as 2 derivatives per line search. (If rich is set to 1, it does 
%  an extra derivative calculation at the beginning of each line search 
%  making a minimum of 3 per line search. Set rich=0 if you think 
%  that the surface is locally quite quadratic.) If the program does average 
%  2 derivatives per line search then it must be superior to most cg methods 
%  including use of Rbackprop (which costs 2 derivatives straight off)
%
%  A possible modification: where the function can be returned at same 
%  time as the dfunction --- there is nothing clever to do with the 
%  value, but it could be used as a sanity check and a convergence criterion. 
%
%  See http://131.111.48.24/mackay/c/macopt.html for further discussion.
%
%  NB: The value of "tol" is totally arbitrary and must be set by 
%  you to a value that works well for your problem. 
%  It depends completely on the typical value of the gradient / step size. 
%
%  Tol specifies a magnitude of gradient at which a halt is called. 
%  or a step size.
%
%  This program MINIMIZES a function.

